import { Query } from 'pretur.sync';
import { Spec, SpecType, Model } from 'pretur.spec';
import { ProviderPool, Transaction } from './pool';
export interface ResolveResult<T extends SpecType> {
    data: Model<T>[];
    count: number;
}
export interface Resolver<T extends SpecType> {
    (transaction: Transaction, query: Query<T>, context?: any): Promise<ResolveResult<T>>;
}
export interface CustomResolver<T extends SpecType> {
    (transaction: Transaction, query: Query<T>, context: any): Promise<ResolveResult<T>>;
}
export interface UnitializedResolver<T extends SpecType> {
    resolver: Resolver<T>;
    initialize(pool: ProviderPool): void;
}
export interface ResolveInterceptor<T extends SpecType> {
    (transaction: Transaction, query: Query<T>, result: ResolveResult<T>, context: any): Promise<ResolveResult<T>>;
}
export interface QueryTransformer<T extends SpecType> {
    (transaction: Transaction, query: Query<T>): Promise<Query<T>>;
}
export interface BuildResolverOptions<T extends SpecType> {
    intercept?: ResolveInterceptor<T>;
    queryTransformer?: QueryTransformer<T>;
}
export declare function buildCustomResolver<T extends SpecType>(_: Spec<T>, resolver: CustomResolver<T>): UnitializedResolver<T>;
export declare function buildResolver<T extends SpecType>(spec: Spec<T>, options?: BuildResolverOptions<T>): UnitializedResolver<T>;
